"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const xlib = tslib_1.__importStar(require("xlib"));
var __ = xlib.lolo;
const dns = require("dns");
/** array of all known disposable email domains, from:https://github.com/yzyjim/disposable-email-domain-list */
// tslint:disable-next-line: no-submodule-imports
const disposableEmailDomains = new Set(require("disposable-email-domain-list/domains.json"));
const resolveMx = __.bb.promisify(dns.resolveMx);
const cachedMx = new xlib.collections.ExpiresMap(__.duration({ minutes: 10 }));
/** returns a report on the validity of the email address (if MX records exist, if the domain is a disposable) */
async function isEmailValid(email, options) {
    options = Object.assign({}, options);
    let split = email.split("@");
    if (split.length !== 2) {
        return { isValid: false, reason: "email invalid format" };
    }
    let domain = split[1].toLowerCase();
    if (disposableEmailDomains.has(domain)) {
        return { isValid: false, reason: "disposable email address" };
    }
    if (options.checkMx === true) {
        try {
            let mxRecords = cachedMx.get(domain);
            if (mxRecords === undefined) {
                mxRecords = await resolveMx(domain);
                if (mxRecords === undefined) {
                    cachedMx.set(domain, null);
                }
                else {
                    cachedMx.set(domain, mxRecords);
                }
            }
            if (mxRecords == null || mxRecords.length === 0) {
                return { isValid: false, reason: `no mxRecord` };
            }
        }
        catch (_err) {
            const err = __.diag.toError(_err);
            return { isValid: false, reason: `invalid mxRecord.  detail=${err.message}` };
        }
    }
    return { isValid: true };
}
exports.isEmailValid = isEmailValid;
// async function _init() {
// }
// const _initPromise = _init();
//# sourceMappingURL=validation.js.map